UNIT VersionCheck;

INTERFACE
TYPE
  TWinVersions    = (vWinUnknown,
                     vWin32S,
                     vWin95,   vWin95OSR2,
                     vWin98,   vWin98SE,
                     vWinME,
                     vWinNT3,  vWinNT4,
                     vWin2000, vWinXP, vWin2003Server,
                     vWinVista, vWin7, vWin8, vWin81,
                     vWin10);
  TOSversions     = (o32bit, o64bit);

CONST
  TWinVersionsStr : Array[TWinVersions] of String[25] =
                    ('Unknown Windows Version',
                     'Windows 32S',
                     'Windows 95',        'Windows 95 OSR2',
                     'Windows 98',        'Windows 98 SE',
                     'Windows Millenium',
                     'Windows NT 3.x',    'Windows NT 4.x',
                     'Windows 2000',      'Windows XP',      'Windows Server 2003',
                     'Windows Vista',     'Windows 7',
                     'Windows 8', 'Windows 8.1',
                     'Windows 10');
  TOSversionsStr  : Array[TOSversions] of String[6] =
                    ('32 Bit', '64 bit');


Function WinVersion : TWinVersions;

Function OSversion : TOSversions;

Function SPinfo : String;


IMPLEMENTATION
USES
  Windows,
  CommCtrl;

{$R VersionCheck.res}
// The resource file attaches the Version manifest to this unit
// and thus to the program.
// A manifest is required since Windows 8.1 to detect the actual
// version number of the OS .......
// The manifest is saved in the file: VersionCheck.manifest
// To create the resource file issue the command:
//        "C:\.......\BRCC32.exe" VersionCheck.rc
// The content of this RC file is just 1 line:
//        1 24 VersionCheck.manifest
//
// The manifest and procedure to add the manifest to a Delphi
// program is described by:
//        Glenn9999, to be found on the Tek-Tip Forums:
//        http://www.tek-tips.com/faqs.cfm?fid=7848

Function SPinfo : String;
Var
  OS : TOSVERSIONINFO;
Begin
  Result                 := '';
  OS.dwPlatformId        := 0;
  OS.dwOSVersionInfoSize := SizeOf(OSVERSIONINFO);
  GetVersionEx(OS);
  Result := OS.szCSDVersion;
End;

Function OSversion : TOSversions;
Type
  TIsWow64Process = function( // Type of IsWow64Process API fn
           Handle  : THandle;
       Var Res     : Boolean) : Boolean; stdcall;
Var       
  IsWow64Process    : TIsWow64Process;  // IsWow64Process fn reference
  IsWow64           : Boolean;
Begin
  IsWow64Process := GetProcAddress(GetModuleHandle('kernel32'), 'IsWow64Process' );
  If Assigned(IsWow64Process) then
    begin
      IsWow64Process(GetCurrentProcess, IsWow64);
      If IsWow64 then Result := o64bit
      else Result := o32bit;
    end
  else Result := o32bit;
end;

Function WinVersion : TWinVersions;
Var
  OS : TOSVERSIONINFO;
Begin
  Result                 := vWinUnknown;
  OS.dwPlatformId        := 0;
  OS.dwOSVersionInfoSize := SizeOf(OSVERSIONINFO);
  GetVersionEx(OS);
  Case OS.dwPlatformId of
     VER_PLATFORM_WIN32_NT:
       begin
         If (OS.dwMajorVersion = 10) AND (OS.dwMinorVersion = 0) then Result := vWin10
         else
         If (OS.dwMajorVersion = 6) AND (OS.dwMinorVersion = 3) then Result := vWin81
         else
         If (OS.dwMajorVersion = 6) AND (OS.dwMinorVersion = 2) then Result := vWin8
         else
         If (OS.dwMajorVersion = 6) AND (OS.dwMinorVersion = 1) then Result := vWin7
         else
         If (OS.dwMajorVersion = 6) AND (OS.dwMinorVersion = 0) then Result := vWinVista
         else
         If (OS.dwMajorVersion = 5) AND (OS.dwMinorVersion = 2) then Result := vWin2003Server
         else
         If (OS.dwMajorVersion = 5) AND (OS.dwMinorVersion = 1) then Result := vWinXP
         else
         If (OS.dwMajorVersion = 5) AND (OS.dwMinorVersion = 0) then Result := vWin2000
         else
         If (OS.dwMajorVersion = 4) then Result := vWinNT4
         else
         If (OS.dwMajorVersion < 4) then Result := vWinNT3
         else;
       end;
     VER_PLATFORM_WIN32_WINDOWS:
       begin
         If (OS.dwMajorVersion = 4) AND (OS.dwMinorVersion = 0) then
           If (OS.szCSDVersion[1] = 'C') OR (OS.szCSDVersion[1] = 'B') then Result := vWin95OSR2
           else Result := vWin95
         else
         If (OS.dwMajorVersion = 4) AND (OS.dwMinorVersion = 10) then
           If (OS.szCSDVersion[1] = 'A') then Result := vWin98SE
           else Result := vWin98
         else
         If (OS.dwMajorVersion = 4) AND (OS.dwMinorVersion = 90) then Result := vWinME
         else;
       end;
     VER_PLATFORM_WIN32s:
       Result := vWin32S;
  end;
End;

INITIALIZATION
  // this call is necessary for the newer "theme" controls.  Note YMMV on Common Controls 6.0
  // ("theme controls") and you may require other corrective changes in your programs for
  // those to work (Glenn9999)
  InitCommonControls;
END.
